@@ -5,10 +5,10 @@
 
 #pragma once
 
-#include <spdlog/formatter.h>
-#include <spdlog/details/log_msg.h>
-#include <spdlog/details/os.h>
-#include <spdlog/fmt/fmt.h>
+#include "spdlog/formatter.h"
+#include "spdlog/details/log_msg.h"
+#include "spdlog/details/os.h"
+#include "spdlog/fmt/fmt.h"
 
 #include <chrono>
 #include <ctime>
@@ -79,60 +79,42 @@ static int to12h(const tm& t)
 }
 
 //Abbreviated weekday name
-using days_array = std::array<std::string, 7>;
-static const days_array& days()
-{
-    static const days_array arr{ { "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" } };
-    return arr;
-}
+static const std::string days[] { "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" };
 class a_formatter:public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm& tm_time) override
     {
-        msg.formatted << days()[tm_time.tm_wday];
+        msg.formatted << days[tm_time.tm_wday];
     }
 };
 
 //Full weekday name
-static const days_array& full_days()
-{
-    static const days_array arr{ { "Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday" } };
-    return arr;
-}
+static const std::string full_days[] { "Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday" };
 class A_formatter:public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm& tm_time) override
     {
-        msg.formatted << full_days()[tm_time.tm_wday];
+        msg.formatted << full_days[tm_time.tm_wday];
     }
 };
 
 //Abbreviated month
-using months_array = std::array<std::string, 12>;
-static const months_array& months()
-{
-    static const months_array arr{ { "Jan", "Feb", "Mar", "Apr", "May", "June", "July", "Aug", "Sept", "Oct", "Nov", "Dec" } };
-    return arr;
-}
+static const std::string  months[] { "Jan", "Feb", "Mar", "Apr", "May", "June", "July", "Aug", "Sept", "Oct", "Nov", "Dec" };
 class b_formatter:public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm& tm_time) override
     {
-        msg.formatted << months()[tm_time.tm_mon];
+        msg.formatted << months[tm_time.tm_mon];
     }
 };
 
 //Full month name
-static const months_array& full_months()
-{
-    static const months_array arr{ { "January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December" } };
-    return arr;
-}
+static const std::string full_months[] { "January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December" };
 class B_formatter:public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm& tm_time) override
     {
-        msg.formatted << full_months()[tm_time.tm_mon];
+        msg.formatted << full_months[tm_time.tm_mon];
     }
 };
 
@@ -153,18 +135,18 @@ static fmt::MemoryWriter& pad_n_join(fmt::MemoryWriter& w, int v1, int v2, int v
 
 
 //Date and time representation (Thu Aug 23 15:35:46 2014)
-class c_formatter:public flag_formatter
+class c_formatter SPDLOG_FINAL:public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm& tm_time) override
     {
-        msg.formatted << days()[tm_time.tm_wday] << ' ' << months()[tm_time.tm_mon] << ' ' << tm_time.tm_mday << ' ';
+        msg.formatted << days[tm_time.tm_wday] << ' ' << months[tm_time.tm_mon] << ' ' << tm_time.tm_mday << ' ';
         pad_n_join(msg.formatted, tm_time.tm_hour, tm_time.tm_min, tm_time.tm_sec, ':') << ' ' << tm_time.tm_year + 1900;
     }
 };
 
 
 // year - 2 digit
-class C_formatter:public flag_formatter
+class C_formatter SPDLOG_FINAL:public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm& tm_time) override
     {
@@ -175,7 +157,7 @@ class C_formatter:public flag_formatter
 
 
 // Short MM/DD/YY date, equivalent to %m/%d/%y 08/23/01
-class D_formatter:public flag_formatter
+class D_formatter SPDLOG_FINAL:public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm& tm_time) override
     {
@@ -185,7 +167,7 @@ class D_formatter:public flag_formatter
 
 
 // year - 4 digit
-class Y_formatter:public flag_formatter
+class Y_formatter SPDLOG_FINAL:public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm& tm_time) override
     {
@@ -194,7 +176,7 @@ class Y_formatter:public flag_formatter
 };
 
 // month 1-12
-class m_formatter:public flag_formatter
+class m_formatter SPDLOG_FINAL:public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm& tm_time) override
     {
@@ -203,7 +185,7 @@ class m_formatter:public flag_formatter
 };
 
 // day of month 1-31
-class d_formatter:public flag_formatter
+class d_formatter SPDLOG_FINAL:public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm& tm_time) override
     {
@@ -212,7 +194,7 @@ class d_formatter:public flag_formatter
 };
 
 // hours in 24 format  0-23
-class H_formatter:public flag_formatter
+class H_formatter SPDLOG_FINAL:public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm& tm_time) override
     {
@@ -221,7 +203,7 @@ class H_formatter:public flag_formatter
 };
 
 // hours in 12 format  1-12
-class I_formatter:public flag_formatter
+class I_formatter SPDLOG_FINAL:public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm& tm_time) override
     {
@@ -230,7 +212,7 @@ class I_formatter:public flag_formatter
 };
 
 // minutes 0-59
-class M_formatter:public flag_formatter
+class M_formatter SPDLOG_FINAL:public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm& tm_time) override
     {
@@ -239,7 +221,7 @@ class M_formatter:public flag_formatter
 };
 
 // seconds 0-59
-class S_formatter:public flag_formatter
+class S_formatter SPDLOG_FINAL:public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm& tm_time) override
     {
@@ -248,7 +230,7 @@ class S_formatter:public flag_formatter
 };
 
 // milliseconds
-class e_formatter:public flag_formatter
+class e_formatter SPDLOG_FINAL:public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm&) override
     {
@@ -259,7 +241,7 @@ class e_formatter:public flag_formatter
 };
 
 // microseconds
-class f_formatter:public flag_formatter
+class f_formatter SPDLOG_FINAL:public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm&) override
     {
@@ -270,7 +252,7 @@ class f_formatter:public flag_formatter
 };
 
 // nanoseconds
-class F_formatter:public flag_formatter
+class F_formatter SPDLOG_FINAL:public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm&) override
     {
@@ -281,7 +263,7 @@ class F_formatter:public flag_formatter
 };
 
 // AM/PM
-class p_formatter:public flag_formatter
+class p_formatter SPDLOG_FINAL:public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm& tm_time) override
     {
@@ -291,7 +273,7 @@ class p_formatter:public flag_formatter
 
 
 // 12 hour clock 02:55:02 pm
-class r_formatter:public flag_formatter
+class r_formatter SPDLOG_FINAL:public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm& tm_time) override
     {
@@ -300,7 +282,7 @@ class r_formatter:public flag_formatter
 };
 
 // 24-hour HH:MM time, equivalent to %H:%M
-class R_formatter:public flag_formatter
+class R_formatter SPDLOG_FINAL:public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm& tm_time) override
     {
@@ -309,7 +291,7 @@ class R_formatter:public flag_formatter
 };
 
 // ISO 8601 time format (HH:MM:SS), equivalent to %H:%M:%S
-class T_formatter:public flag_formatter
+class T_formatter SPDLOG_FINAL:public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm& tm_time) override
     {
@@ -317,14 +299,13 @@ class T_formatter:public flag_formatter
     }
 };
 
-
 // ISO 8601 offset from UTC in timezone (+-HH:MM)
-class z_formatter:public flag_formatter
+class z_formatter SPDLOG_FINAL:public flag_formatter
 {
 public:
     const std::chrono::seconds cache_refresh = std::chrono::seconds(5);
 
-    z_formatter():_last_update(std::chrono::seconds(0))
+    z_formatter():_last_update(std::chrono::seconds(0)), _offset_minutes(0)
     {}
     z_formatter(const z_formatter&) = delete;
     z_formatter& operator=(const z_formatter&) = delete;
@@ -376,7 +357,7 @@ private:
 
 
 // Thread id
-class t_formatter:public flag_formatter
+class t_formatter SPDLOG_FINAL:public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm&) override
     {
@@ -385,7 +366,7 @@ class t_formatter:public flag_formatter
 };
 
 // Current pid
-class pid_formatter:public flag_formatter
+class pid_formatter SPDLOG_FINAL:public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm&) override
     {
@@ -394,7 +375,7 @@ class pid_formatter:public flag_formatter
 };
 
 
-class v_formatter:public flag_formatter
+class v_formatter SPDLOG_FINAL:public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm&) override
     {
@@ -402,7 +383,7 @@ class v_formatter:public flag_formatter
     }
 };
 
-class ch_formatter:public flag_formatter
+class ch_formatter SPDLOG_FINAL:public flag_formatter
 {
 public:
     explicit ch_formatter(char ch): _ch(ch)
@@ -417,7 +398,7 @@ private:
 
 
 //aggregate user chars to display as is
-class aggregate_formatter:public flag_formatter
+class aggregate_formatter SPDLOG_FINAL:public flag_formatter
 {
 public:
     aggregate_formatter()
@@ -436,7 +417,7 @@ private:
 
 // Full info formatter
 // pattern: [%Y-%m-%d %H:%M:%S.%e] [%n] [%l] %v
-class full_formatter:public flag_formatter
+class full_formatter SPDLOG_FINAL:public flag_formatter
 {
     void format(details::log_msg& msg, const std::tm& tm_time) override
     {
@@ -488,7 +469,8 @@ class full_formatter:public flag_formatter
 ///////////////////////////////////////////////////////////////////////////////
 // pattern_formatter inline impl
 ///////////////////////////////////////////////////////////////////////////////
-inline spdlog::pattern_formatter::pattern_formatter(const std::string& pattern)
+inline spdlog::pattern_formatter::pattern_formatter(const std::string& pattern, pattern_time_type pattern_time)
+    : _pattern_time(pattern_time)
 {
     compile_pattern(pattern);
 }
@@ -645,19 +627,32 @@ inline void spdlog::pattern_formatter::handle_flag(char flag)
         _formatters.push_back(std::unique_ptr<details::flag_formatter>(new details::pid_formatter()));
         break;
 
-    default: //Unkown flag appears as is
+#if defined(SPDLOG_ENABLE_MESSAGE_COUNTER)
+    case ('i'):
+        _formatters.push_back(std::unique_ptr<details::flag_formatter>(new details::i_formatter()));
+        break;
+#endif
+
+    default: //Unknown flag appears as is
         _formatters.push_back(std::unique_ptr<details::flag_formatter>(new details::ch_formatter('%')));
         _formatters.push_back(std::unique_ptr<details::flag_formatter>(new details::ch_formatter(flag)));
         break;
     }
 }
 
+inline std::tm spdlog::pattern_formatter::get_time(details::log_msg& msg)
+{
+    if (_pattern_time == pattern_time_type::local)
+        return details::os::localtime(log_clock::to_time_t(msg.time));
+    else
+        return details::os::gmtime(log_clock::to_time_t(msg.time));
+}
 
 inline void spdlog::pattern_formatter::format(details::log_msg& msg)
 {
 
 #ifndef SPDLOG_NO_DATETIME
-    auto tm_time = details::os::localtime(log_clock::to_time_t(msg.time));
+    auto tm_time = get_time(msg);
 #else
     std::tm tm_time;
 #endif
